package com.lantone.structure.facade.tran;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.lantone.common.util.StringUtil;
import com.lantone.structure.facade.tran.util.CommonAnalysisUtil;
import com.lantone.structure.model.doc.ThreeLevelWardDoc;
import com.lantone.structure.model.label.ThreeLevelWardLabel;
import lombok.extern.slf4j.Slf4j;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @Description:上级医师查房记录
 * @author: cy
 * @time: 2021/2/20 14:07
 */
@Slf4j
public class ThreeLevelWardTran extends TargetTran {

    @Override
    public Map<String, String> convert(String text) {
        List<ThreeLevelWardDoc> wardRecordDocList = new ArrayList<>();
        ThreeLevelWardDoc wardRecordDoc = new ThreeLevelWardDoc();
        wardRecordDoc.setText(text);
        wardRecordDocList.add(wardRecordDoc);
        inputInfo.setThreeLevelWardDocs(wardRecordDocList);
        Map<String, String> structureMap = cutWord(text);
        wardRecordContrast(inputInfo.getThreeLevelWardDocs(), structureMap);
        wardRecordDoc.setStructureMap(structureMap);
        aiProcess();
        lastSovle(inputInfo.getThreeLevelWardDocs(),structureMap);
        return structureMap;
    }

    private Map<String, String> cutWord(String text) {
        Map<String, String> sourceMap = Maps.newHashMap();
        List<String> titles = CommonAnalysisUtil.sortTitles(
                Lists.newArrayList("鉴别诊断","诊疗计划","补充诊断"),
                text
        );
        CommonAnalysisUtil.cutByTitles(text, titles, 0, sourceMap);
        if(sourceMap.containsKey("鉴别诊断") || sourceMap.containsKey("补充诊断")){
            sourceMap.remove("鉴别诊断");
            sourceMap.remove("补充诊断");
        }
        return sourceMap;
    }


    public void wardRecordContrast( List<ThreeLevelWardDoc> wardRecordDoc,Map<String, String> retMap) {
        String text = wardRecordDoc.get(0).getText();
        //查房日期时间
        String sumdDate = extractDate(text);
        if(StringUtil.isNotEmpty(sumdDate)){
            retMap.put("查房日期时间",sumdDate);
        }
        if(text.contains("鉴别诊断")){
            String logo = text.split("\n")[1];
            String fir = text.substring(text.indexOf(logo) + logo.length(),text.lastIndexOf("鉴别诊断"));
            if(StringUtil.isNotEmpty(fir)){
                retMap.put("查房记录",fir);
            }
        }

        if(StringUtil.isNotEmpty(retMap.get("诊疗计划"))){
            String plan = retMap.get("诊疗计划");
               String Data = parseString(plan);
               retMap.put("诊疗计划",Data);
           }
        }

    public static String parseString(String text){
        if(text.contains("/")){
            text = text.substring(0,text.lastIndexOf("/"));
            text= parseString(text);
        }
        return text;
    }

    /**
     * 抽取文本中的第一个时间
     *
     * @param top
     * @return
     */
    public static String extractDate(String top) {
        Pattern pattern = Pattern.compile("[0-9]{4}[-][0-9]{1,2}[-][0-9]{1,2}");
        Matcher matcher = pattern.matcher(top);
        if (matcher.find()) {
            return matcher.group(0);
        } else {
            Pattern p1 = Pattern.compile("[0-9]{4}年[0-9]+月[0-9]+日");
            Matcher m1 = p1.matcher(top);
            if (m1.find()) {
                return m1.group(0);
            }
        }
        return null;
    }

    public static void lastSovle(List<ThreeLevelWardDoc> threeLevelWardDocs, Map<String,String> structureMap) {
        if(!threeLevelWardDocs.isEmpty() && threeLevelWardDocs.get(0)!=null && threeLevelWardDocs.get(0)
                .getThreeLevelWardLabel()!=null){
            ThreeLevelWardLabel threeLevelWardLabel = threeLevelWardDocs.get(0).getThreeLevelWardLabel().get(0);
            String doctorAdviceRecord = threeLevelWardLabel.getDoctorAdviceRecord();
            if(StringUtil.isNotEmpty(doctorAdviceRecord)){
                structureMap.put("医嘱内容",doctorAdviceRecord);
            }
        }
    }
}